import django
django.setup()

from json import dumps

from django.contrib.auth.models import User
from django.test import TestCase

from perestroika.exceptions import BadRequest
from perestroika.utils import dict_to_multi_dict


class DjangoTest(TestCase):
    def make_post(self, url, data):
        return self.client.post(url, dumps(data), content_type='application/json')

    def make_put(self, url, data):
        return self.client.put(url, dumps(data), content_type='application/json')

    def make_empty_post(self, url):
        return self.client.post(url, content_type='application/json')

    def make_get(self, url, data):
        data = dict_to_multi_dict(data)
        return self.client.get(url, data, content_type='application/json')

    def test_allowed_methods(self):
        _response = self.make_post("/test/empty/", {})
        assert _response.status_code == 405

    def test_empty_get(self):
        User(username="first").save()

        assert User.objects.count() == 1

        _response = self.make_get("/test/full/", {})
        assert _response.status_code == 200
        assert _response.json() == {
            'items': [{'username': "first"}],
            'status_code': 200,
            'total': 1
        }

        User(username="second").save()

        assert User.objects.count() == 2

        _response = self.make_get("/test/full/", {})
        assert _response.status_code == 200
        assert _response.json() == {
            'items': [{'username': "first"}, {"username": "second"}],
            'status_code': 200,
            'total': 2
        }

    def test_json_validation_no_items(self):
        with self.assertRaises(BadRequest):
            _response = self.make_empty_post("/test/full/")

    def test_post(self):
        assert User.objects.count() == 0

        _response = self.make_post("/test/full/", {'items': [{'username': "third"}]})
        assert _response.status_code == 201
        assert _response.json() == {
            'status_code': 201,
            'created': 1
        }

    def test_put(self):
        assert User.objects.count() == 0

        _response = self.make_post("/test/full/", {'items': [{'username': "third"}]})
        assert User.objects.count() == 1

        _response = self.make_put("/test/full/", {'items': [{'username': "fourth"}]})
        assert User.objects.count() == 1

        assert User.objects.all().first().username == "fourth"

    def test_admin(self):
        _response = self.make_get("/admin/", {})
        assert _response.status_code in [200, 302]

    def test_filter_get(self):
        User(username="first").save()
        User(username="second").save()
        User(username="third").save()

        assert User.objects.count() == 3

        _response = self.make_get("/test/full/", {'filter.username__in': 'first'})
        assert _response.status_code == 200

        assert _response.json() == {
            'items': [{'username': "first"}],
            'filter': {'username__in': ['first']},
            'status_code': 200,
            'total': 1
        }

    def test_exclude_get(self):
        User(username="first").save()
        User(username="second").save()
        User(username="third").save()

        assert User.objects.count() == 3

        _response = self.make_get("/test/full/", {'exclude.username__in': 'first'})
        assert _response.status_code == 200

        assert _response.json() == {
            'items': [{'username': "second"}, {'username': "third"}],
            'exclude': {'username__in': ['first']},
            'status_code': 200,
            'total': 2
        }

    def test_filter_put(self):
        User(username="first").save()
        User(username="second").save()
        User(username="third").save()

        assert User.objects.count() == 3

        _response = self.make_put("/test/full/", {'items': [{'username': 'fourth'}], 'filter.username': 'first'})
        assert _response.json() == {
            'filter': {'username': 'first'},
            'updated': 1,
            'status_code': 200,
        }
        assert _response.status_code == 200

        _response = self.make_get("/test/full/", {})
        assert _response.status_code == 200

        assert _response.json() == {
            'items': [{'username': "fourth"}, {'username': "second"}, {'username': "third"}],
            'status_code': 200,
            'total': 3
        }

    def test_exclude_put(self):
        User(username="first").save()
        User(username="second").save()
        User(username="third").save()

        assert User.objects.count() == 3

        _response = self.make_put("/test/full/", {'items': [{'username': 'fourth'}],
                                                  'exclude.username__in': ['first', 'second']})
        assert _response.json() == {
            'exclude': {'username__in': ['first', 'second']},
            'updated': 1,
            'status_code': 200,
        }
        assert _response.status_code == 200

        _response = self.make_get("/test/full/", {})
        assert _response.status_code == 200

        assert _response.json() == {
            'items': [{'username': "first"}, {'username': "second"}, {'username': "fourth"}],
            'status_code': 200,
            'total': 3
        }



from typing import Any, List, Union

from django.db.models import Q, QuerySet
from validate_it import schema, Options


@schema
class Context:
    request: Any
    queryset: Any

    order: dict = Options(default=dict)

    user_filter: dict = Options(default=dict)
    user_exclude: dict = Options(default=dict)

    system_filter: Union[dict, Q, None] = Options(default=None)
    system_exclude: Union[dict, Q, None] = Options(default=None)

    merged_filter: Union[dict, Q, None] = Options(default=None)
    merged_exclude: Union[dict, Q, None] = Options(default=None)

    project: List[str] = Options(default=list)
    items: Union[List[dict], QuerySet] = Options(default=list)
    meta: dict = Options(default=dict)
    status_code: int = Options(default=0)

    limit: int = Options(default=0)
    skip: int = Options(default=0)

    total: int = Options(default=0)
    created: int = Options(default=0)
    updated: int = Options(default=0)
    deleted: int = Options(default=0)



import json
from typing import Dict, Union, Optional

from perestroika.exceptions import RestException
from perestroika.methods import Method


class Resource:
    methods: Optional[Dict[str, Method]] = None

    def handler(self, request, **kwargs):
        raise NotImplementedError()

    def get_method_handler(self, **kwargs):
        raise NotImplementedError()

    def method_not_permitted(self):
        raise NotImplementedError()


class DjangoResource(Resource):
    from django.views.decorators.csrf import csrf_exempt

    cache_control: Dict[str, Union[bool, int]] = None

    def get_method_handler(self, **kwargs):
        request = kwargs['request']

        if self.methods:
            return self.methods.get(request.method.lower())

    def method_not_permitted(self):
        from django.http import HttpResponseNotAllowed

        permitted_methods = list(self.methods.keys()) if self.methods else []
        return HttpResponseNotAllowed(permitted_methods=permitted_methods)

    @csrf_exempt
    def handler(self, request, **kwargs):
        from django.utils.cache import patch_cache_control

        method_handler = self.get_method_handler(request=request)

        if method_handler:
            response = method_handler.handle(request)

            if self.cache_control:
                patch_cache_control(response, **self.cache_control)

            return response

        return self.method_not_permitted()

    def schema(self, request):
        from django.http import JsonResponse
        _schema = {
            k: v.schema() for k, v in self.methods.items()
        }

        return JsonResponse(_schema)


class JSONResource(Resource):
    def get_method_handler(self, **kwargs):
        if self.methods:
            return self.methods.get(kwargs.get('method'))

    def method_not_permitted(self):
        permitted_methods = list(self.methods.keys()) if self.methods else []

        return json.dumps(
            {
                'status': 'error',
                'status_code': 405,
                'message': f'Permitted methods: {permitted_methods}'
            }
        )

    def handler(self, request, **kwargs):
        method_handler = self.get_method_handler(**kwargs)

        if method_handler:
            try:
                response = method_handler.handle(
                    request,
                    json_data=kwargs.get('json_data'),
                    method=kwargs.get('method')
                )
            except RestException as e:
                return json.dumps(
                    {
                        'status': 'error',
                        'status_code': e.status_code,
                        'message': e.message
                    }
                )

            return response

        return self.method_not_permitted()



from django.contrib.auth.models import User

from perestroika.methods import Get, Post, Put, Delete
from perestroika.resource import DjangoResource
from perestroika.validators import AllowAll


class EmptyResource(DjangoResource):
    pass


class OutUser:
    def __call__(self, item):
        return {"username": item["username"]}


class FullResource(DjangoResource):
    cache_control = dict(max_age=0, no_cache=True, no_store=True, must_revalidate=True)

    get = Get(
        queryset=User.objects.all(),
        output_validator=OutUser(),
        count_total=True
    )

    post = Post(
        queryset=User.objects.all(),
        input_validator=AllowAll(),
        output_validator=OutUser()
    )

    put = Put(
        queryset=User.objects.all(),
        input_validator=AllowAll(),
        output_validator=OutUser()
    )

    delete = Delete(
        queryset=User.objects.all()
    )


__all__ = [
    "EmptyResource",
    "FullResource",
]



from django.db.models import Q

from perestroika.context import Context
from perestroika.exceptions import BadRequest


class DbLayer:
    pass


class DjangoDbLayer(DbLayer):
    @classmethod
    def queryset_to_items(cls, context):
        return context.queryset.values()

    @classmethod
    def merge_filters(cls, context: Context):
        query = Q()

        if context.user_filter:
            query &= Q(**context.user_filter)

        if context.system_filter:
            query &= context.system_filter

        context.merged_filter = query

    @classmethod
    def merge_excludes(cls, context: Context):
        query = Q()

        if context.user_exclude:
            query &= Q(**context.user_exclude)

        if context.system_exclude:
            query &= context.system_exclude

        context.merged_exclude = query

    @classmethod
    def get(cls, context: Context, method):
        cls.merge_filters(context)
        cls.merge_excludes(context)

        context.queryset = context.queryset.filter(context.merged_filter)
        context.queryset = context.queryset.exclude(context.merged_exclude)

        if context.project:
            context.queryset = context.queryset.only(*context.project)

        context.items = cls.queryset_to_items(context)

        if method.count_total:
            context.total = context.queryset.count()

    @staticmethod
    def post(context: Context, method):
        if not context.items:
            raise BadRequest(message="Empty data for insert")

        items = [context.queryset.model(**data) for data in context.items]

        context.queryset.model.objects.bulk_create(items)
        context.created = len(items)

    @classmethod
    def put(cls, context: Context, method):
        if not context.items:
            raise BadRequest(message="Empty data for update")

        cls.merge_filters(context)
        cls.merge_excludes(context)

        context.queryset = context.queryset.filter(context.merged_filter)
        context.queryset = context.queryset.exclude(context.merged_exclude)

        updated = 0

        for item in context.items:
            updated += context.queryset.update(**item)

        context.updated = updated



import logging
from collections import Callable
from typing import List, Any, Optional
from validate_it import schema, Options

from perestroika.context import Context
from perestroika.db_layers import DbLayer, DjangoDbLayer
from perestroika.deserializers import Deserializer, DjangoDeserializer
from perestroika.exceptions import RestException, BadRequest, InternalServerError
from perestroika.serializers import Serializer, DjangoSerializer
from perestroika.validators import DenyAll

logger = logging.getLogger(__name__)


@schema
class Method:
    mode: str = 'django'

    query: Any = None
    queryset: Any = None
    db_layer: Optional[DbLayer] = None

    serializer: Optional[Serializer] = None
    deserializer: Optional[Deserializer] = None

    skip_query_db: bool = Options(default=False)

    input_validator: Callable = Options(default=DenyAll)
    output_validator: Callable = Options(default=DenyAll)

    pre_query_hooks: List[Callable] = Options(default=list)
    post_query_hooks: List[Callable] = Options(default=list)

    request_hooks: List[Callable] = Options(default=list)
    response_hooks: List[Callable] = Options(default=list)

    def __validate_it__post_init__(self):
        need_fields = []

        if self.mode == 'django':
            need_fields = ['queryset']

            if not self.db_layer:
                self.db_layer = DjangoDbLayer()

            if not self.serializer:
                self.serializer = DjangoSerializer()

            if not self.deserializer:
                self.deserializer = DjangoDeserializer()

        for field in need_fields:
            if getattr(self, field) is None and not self.skip_query_db:
                raise ValueError(f"Empty `{field}` is allowed only for resources with `skip_query_db` == True")

    def __set_name__(self, owner, name):
        if not hasattr(owner, 'methods') or not owner.methods:
            setattr(owner, 'methods', {})

        owner.methods[self.__class__.__name__.lower()] = self

    def schema(self):
        return {
            "output_schema": repr(self.output_validator)
        }

    def query_db(self, context: Context):
        raise NotImplementedError()

    @staticmethod
    def validate(validator, context: Context, validation_exception_class=RestException):
        _errors = []
        items = []

        for item in context.items:
            try:
                item = validator(item)
                items.append(item)
            except Exception as e:
                _desc = {
                    "data": item,
                    "error": e
                }
                _errors.append(e)

        if _errors:
            raise validation_exception_class(message="Wrong data", errors=_errors)

        context.items = items

    def validate_input(self, context: Context):
        self.validate(self.input_validator, context, validation_exception_class=BadRequest)

    def validate_output(self, context: Context):
        self.validate(self.output_validator, context, validation_exception_class=InternalServerError)

    @staticmethod
    def apply_hooks(hooks, context: Context):
        for hook in hooks:
            hook(context)

    def apply_pre_query_hooks(self, context: Context):
        self.apply_hooks(self.pre_query_hooks, context)

    def apply_post_query_hooks(self, context: Context):
        self.apply_hooks(self.post_query_hooks, context)

    def set_default_success_code(self, context: Context):
        raise NotImplementedError()

    def apply_request_hooks(self, context: Context):
        self.apply_hooks(self.request_hooks, context)

    def apply_response_hooks(self, context: Context):
        self.apply_hooks(self.response_hooks, context)

    def handle(self, request, **kwargs):
        context = self.deserializer.deserialize(request, self, **kwargs)

        self.set_default_success_code(context)
        self.apply_request_hooks(context)
        self.validate_input(context)
        self.apply_pre_query_hooks(context)

        if not self.skip_query_db:
            self.query_db(context)

        self.apply_post_query_hooks(context)
        self.validate_output(context)
        self.apply_response_hooks(context)

        return self.serializer.serialize(context)


@schema
class CanFilterAndExclude(Method):
    filter_validator: Callable = Options(default=DenyAll)
    exclude_validator: Callable = Options(default=DenyAll)

    def set_default_success_code(self, context: Context):
        raise NotImplementedError()

    def query_db(self, context: Context):
        raise NotImplementedError()

    def schema(self):
        _schema = super().schema()
        _schema["filter_schema"] = repr(self.filter_validator)
        _schema["exclude_schema"] = repr(self.exclude_validator)
        return _schema


@schema
class NoBodyNoObjectsNoInput(CanFilterAndExclude):
    def set_default_success_code(self, context: Context):
        raise NotImplementedError()

    def validate_input(self, context: Context):
        """ Void validation because no input data"""
        pass

    def query_db(self, context: Context):
        raise NotImplementedError()


@schema
class Get(NoBodyNoObjectsNoInput):
    count_total: bool = Options(default=False)

    def query_db(self, context: Context):
        self.db_layer.get(context, self)

    def set_default_success_code(self, context: Context):
        context.status_code = 200


@schema
class Post(Method):
    input_validator: Callable = Options(default=DenyAll)

    def query_db(self, context: Context):
        self.db_layer.post(context, self)
        context.items = []

    def schema(self):
        _schema = super().schema()
        _schema["input_schema"] = repr(self.input_validator)
        return _schema

    def set_default_success_code(self, context: Context):
        context.status_code = 201


@schema
class Put(CanFilterAndExclude):
    def query_db(self, context: Context):
        self.db_layer.put(context, self)
        context.items = []

    input_validator: Callable = Options(default=DenyAll)

    def set_default_success_code(self, context: Context):
        context.status_code = 200

    def schema(self):
        _schema = super().schema()
        _schema["input_schema"] = repr(self.input_validator)
        return _schema


@schema
class Delete(NoBodyNoObjectsNoInput):
    def query_db(self, context: Context):
        raise NotImplementedError()

    def set_default_success_code(self, context: Context):
        context.status_code = 204


__all__ = [
    "Get",
    "Post",
    "Put",
    "Delete"
]



import json

from django.core.serializers.json import DjangoJSONEncoder
from django.http import JsonResponse

from perestroika.context import Context


class Serializer:
    def serialize(self, context):
        raise NotImplementedError()

    def clean_item(self, item, project):
        to_drop = [
            k
            for k in item.keys()
            if k not in project
        ]

        for k in to_drop:
            del item[k]

    def apply_project(self, context: Context):
        if context.project:
            for item in context.items:
                self.clean_item(item, context.project)

    def get_data(self, context: Context):
        _data = {}

        self.apply_project(context)

        for name in [
            'items',

            'order',

            'project',
            'status_code',

            'limit', 'skip',

            'total', 'created', 'updated', 'deleted',
        ]:
            value = getattr(context, name)
            if value:
                _data[name] = getattr(context, name)

        if context.user_filter:
            _data['filter'] = context.user_filter

        if context.user_exclude:
            _data['exclude'] = context.user_exclude

        return _data


class DjangoSerializer(Serializer):
    @staticmethod
    def get_encoder():
        return DjangoJSONEncoder

    def serialize(self, context: Context):
        _data = self.get_data(context)

        return JsonResponse(_data, status=context.status_code, encoder=self.get_encoder())


class JSONSerializer(Serializer):
    @staticmethod
    def get_encoder():
        return json.JSONEncoder

    def serialize(self, context: Context):
        _data = self.get_data(context)

        return json.dumps(_data, cls=self.get_encoder())



class AllowAll:
    def __call__(self, item):
        return item


class DenyAll:
    def __call__(self, item):
        raise TypeError("Deny all types")


# noinspection PyBroadException
class AnyMatch:
    def __init__(self, *validators):
        self.validators = validators

    def __call__(self, item):
        for validator in self.validators:
            try:
                return validator(item)
            except Exception:
                continue
        else:
            raise TypeError("No matches found")


class AllMatch:
    def __init__(self, *validators):
        self.validators = validators

    def __call__(self, item):
        for validator in self.validators:
            item = validator(item)

        return item



from django.contrib import admin
from django.urls import path, include

from test_django.urls import urlpatterns as test_urls


urlpatterns = [
    path('admin/', admin.site.urls),
    path('test/', include(test_urls)),
]



from django.utils.datastructures import MultiValueDict


def dict_to_multi_dict(data):
    return MultiValueDict(data)


def multi_dict_to_dict(query_dict):
    _data = {}
    query_dict = query_dict.copy()
    keys = list(query_dict.keys())

    for key in keys:
        _data[key] = query_dict.pop(key)

    return _data



import os
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = 'randomsecret'
DEBUG = True

ALLOWED_HOSTS = [
    "localhost",
    "testserver",
]

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'config.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

LANGUAGE_CODE = 'en'

from django.utils.translation import ugettext_lazy as _

LANGUAGES = (
    ('en', _("English")),
    ('ru', _("Russian")),
)

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'
STATIC_ROOT = 'static'



from unittest import TestCase

from perestroika.validators import AllowAll, DenyAll, AnyMatch, AllMatch


class ValidatorsTest(TestCase):
    def test_allow_all(self):
        validator = AllowAll()
        validator(1)
        validator('z')
        validator(None)

    def test_deny_all(self):
        validator = DenyAll()

        with self.assertRaises(TypeError):
            validator(1)

        with self.assertRaises(TypeError):
            validator('z')

        with self.assertRaises(TypeError):
            validator(None)

    def test_any_match(self):
        validator = AnyMatch(AllowAll(), DenyAll())
        validator(1)

        validator = AnyMatch(DenyAll(), AllowAll())
        validator(1)

    def test_all_match(self):
        validator = AllMatch(AllowAll(), DenyAll())

        with self.assertRaises(TypeError):
            validator(1)

        with self.assertRaises(TypeError):
            validator('z')

        with self.assertRaises(TypeError):
            validator(None)

        validator = AllMatch(DenyAll(), AllowAll())

        with self.assertRaises(TypeError):
            validator(1)

        with self.assertRaises(TypeError):
            validator('z')

        with self.assertRaises(TypeError):
            validator(None)

        validator = AllMatch(AllowAll(), AllowAll())
        validator(1)
        validator('z')
        validator(None)



class RestException(Exception):
    def __init__(self, message=None, status_code=None, errors=None) -> None:
        self.message = message
        self.status_code = status_code
        self.errors = errors or []

    def __str__(self):
        return f"{self.message}: {self.errors}"


class BadRequest(RestException):
    def __init__(self, message=None, errors=None) -> None:
        super().__init__(message, 400, errors)


class InternalServerError(RestException):
    def __init__(self, message=None, errors=None) -> None:
        super().__init__(message, 500, errors)


class MethodNotAllowed(RestException):
    def __init__(self, message=None, errors=None) -> None:
        super().__init__(message, 405, errors)



from os import path

from setuptools import setup, find_packages

version = "0.1.19"

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="perestroika",
    packages=find_packages(),
    version=version,
    description="Rest lib",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Ruslan Roskoshnyj",
    author_email="i.am.yarger@gmail.com",
    url="https://github.com/newmediatech/perestroika",
    download_url="https://github.com/newmediatech/perestroika/archive/{}.tar.gz".format(version),
    keywords=["REST"],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django :: 2.2',
    ],
    python_requires=">3.6.0",
    platforms=["OS Independent"],
    license="LICENSE.txt",
    install_requires=[
        "validate-it (>=0.10.0)",
        "accordion (>=0.2.3)",
        "django (>=2.2.3)",
    ],
    extras_require={
        "tests": [
            "pytest (==3.6.0)",
            "coverage (==4.5)",
            "pytest-cov (==2.5.1)",
            "django (==2.2)",
            "pytest-django (==3.2.0)",
        ],
        "docs": []
    }
)



import json
from json import JSONDecodeError

from accordion import expand

from perestroika.context import Context
from perestroika.exceptions import BadRequest
from perestroika.utils import multi_dict_to_dict


class Deserializer:
    def get_data(self, request, method_handler, **kwargs):
        raise NotImplementedError()

    def deserialize(self, request, method_handler, **kwargs):
        data = self.get_data(request, method_handler, **kwargs)

        if not data.get("items") and request.method.lower() in ["post", "put", "patch"]:
            raise BadRequest(message="Need data for processing")

        return Context(
            request=request,

            order=data.get("order", {}),

            user_filter=data.get("filter", {}),
            user_exclude=data.get("exclude", {}),

            project=data.get("project", []),

            items=data.get("items", []),
            queryset=method_handler.queryset,
            meta=data.get("meta", {}),
        )


class DjangoDeserializer(Deserializer):
    def get_data(self, request, method_handler, **kwargs):
        try:
            data = json.loads(request.body)
            data = expand(data)
        except JSONDecodeError:
            data = {}

        get_data = multi_dict_to_dict(request.GET)
        get_data = expand(get_data)
        data.update(get_data)

        post_data = multi_dict_to_dict(request.POST)
        post_data = expand(post_data)
        data.update(post_data)

        return data


class JSONDeserializer(Deserializer):
    def get_data(self, request, method_handler, **kwargs):
        return kwargs.get("json_data", {})
